/**
 * Déclenche une opération asynchronise depuis un bouton, devant le mettre à jour !
 * 
 * @param {*} elt 
 * @param {*} url 
 */
async function lcvprestaconnector_ajaxButtonReactionCall(elt, url) {

    // On commence par remplacer le texte du bouton par un loader
    // On cache l'élément (display : none)
    const before = elt.style.display;
    elt.style.display = 'none';

    // On ajoute un loader qu'on affiche juste à côté de l'élément
    const loader = document.createElement('div');
    loader.className = 'spinner';
    loader.style.width = '18px';
    loader.style.height = '18px';

    // Insertion du loader juste après l'élement
    elt.parentNode.insertBefore(loader, elt.nextSibling);    

    try
    {
        const baseUrl = $('body').data('base-url');
        const token = $('body').data('token');

        // Appel ajax pour synchroniser le produit
        const response = await fetch(`${baseUrl}/modules/lcvprestaconnector/${url}&_token=${token}`, {
            method: 'GET',
            JSON: true,
            headers: {
                'Content-Type': 'application/json'
            }
        });        

        if (!response.ok)
            throw new Error('Erreur HTTP : ' + response);

        // Normalement, la réponse devrait contenir un JSON avec une propriété 'success' à vrai
        // Si ce n'est pas le cas, on lève une erreur
        const data = await response.json();
        if (data.success)
        {
            return true;
        }
        else
        {
            console.error(data);
            elt.innerHTML = 'Erreur : ' + data.message;        
            return false;
        }
    }
    catch (e)
    {
        console.error(e);
        elt.innerHTML = 'Erreur !';
    }
    finally
    {
        // On montre l'élément !
        elt.style.display = before;
        // On supprime le loader
        loader.remove();
    }

    return false;
}

/**
 * Déclenche une resynchronisation globale
 */
async function lcvprestaconnector_resyncAll() {
    console.debug('Resynchronisation totale demandée !');

    // Si un bouton avec un id 'page-header-desc-configuration-resync' existe, on le désactive et on change son nom
    const elt = document.getElementById('page-header-desc-configuration-resync');
    
    if (await lcvprestaconnector_ajaxButtonReactionCall(elt, 'resync_all?'))
        elt.innerHTML = 'Resynchro. demandée !';
}

/**
 * Déclenche la synchronisation d'un produit spécifique
 * @param {*} e évènement déclencheur
 * @param {*} id_product 
 * 
 */
async function lcvprestaconnector_syncProduct(e, id_product) {    
    
    console.debug('Synchronisation du produit ' + id_product);
    const elt = e.currentTarget;    

    if (await lcvprestaconnector_ajaxButtonReactionCall(elt, `sync_product?id_product=${id_product}`));
        document.location.reload();
}

/**
 * Déclenche la synchronisation d'un produit spécifique
 * @param {*} e évènement déclencheur
 * @param {*} id_product 
 * 
 */
async function lcvprestaconnector_toggleLockProduct(e, id_product) {    
    
    console.debug('Lock/Unlock produit ' + id_product);
    const elt = e.currentTarget;    
    const action = elt.innerHTML.includes('lock_open') ? 'lock' : 'unlock';

    if (await lcvprestaconnector_ajaxButtonReactionCall(elt, `lock_product?id_product=${id_product}&action=${action}`))
        document.location.reload();
}
